package source
{
	import flash.geom.Point;
	import net.flashpunk.Entity;
	import net.flashpunk.FP;
	import net.flashpunk.utils.Draw;
	import net.flashpunk.utils.Input;
	import net.flashpunk.utils.Key;
	/**
	 * ...
	 * @author Time
	 */
	public class Particle extends Entity
	{
		public var s:Point;
		private var size:int;
		public static const friction:Number = 1;
		public static const gravity:Number = 0.25;
		private var color:uint = 0x00FF00;
		
		public function Particle(_x:int, _y:int, _s:Point, _size:int) 
		{
			super(_x, _y);
			s = _s;
			size = _size;
			setHitbox(size, size);
			type = "Solid";
		}
		
		
		override public function update():void
		{
			s.y += gravity;
			if (Input.mouseDown)
			{
				var mpower:int = 4;
				var mdisMax:int = 100;
				var mdis:Number = FP.distance(x + size / 2, y + size / 2, Input.mouseX, Input.mouseY);
				var mang:Number = FP.angle(x + size / 2, y + size / 2, Input.mouseX, Input.mouseY);
				s.x += mpower * Math.max(0, mdisMax - mdis) / mdisMax * Math.cos((180 - mang) / 180 * Math.PI);
				s.y += mpower * Math.max(0, mdisMax - mdis) / mdisMax * Math.sin((180 - mang) / 180 * Math.PI);
			}
			
			if (Input.check(Key.X))
			{
				if (!Math.floor(Math.random() * 20))
				{
					s.x += Math.random() * 4 - 2;
				}
				if (!Math.floor(Math.random() * 10))
				{
					s.y -= 4;
				}
			}
			
			incrementMovement();
			
			
			if (x < FP.camera.x && !collide("Solid", FP.camera.x, y))
			{
				s.x = 1;
				x = FP.camera.x;
			}
			/*if (y < FP.camera.y && !collide("Solid", x, FP.camera.y))
			{
				s.y = 1;
				y = FP.camera.y;
			}*/
			if (x + size > FP.camera.x + FP.width && !collide("Solid", FP.camera.x + FP.width - size, y))
			{
				s.x = -1;
				x = FP.camera.x + FP.width - size;
			}
			if (y + size > FP.camera.y + FP.height && !collide("Solid", x, FP.camera.y + FP.height - size))
			{
				//s.y = -10;
				y = FP.camera.y + FP.height - size;
			}
			
			s.x *= friction;
			if (!collide("Solid", Math.round(x), Math.round(y)))
			{
				x = Math.round(x);
				y = Math.round(y);
			}
			
			color = 0x00FF00;
			if (collide("Solid", x, y))
			{
				color = 0xFF0000;
			}
		}
		
		public function incrementMovement():void
		{
			if (s.x != 0)
			{
				var useLoop:Boolean = false;
				var ve:Vector.<Entity> = new Vector.<Entity>();
				var xPos:int = 0;
				if (s.x < 0)
				{
					xPos = s.x;
				}
				FP.world.collideRectInto("Solid", x + xPos, y, Math.abs(s.x) + size, size, ve);
				if (ve.length >= 2 || (ve.length > 0 && ve[0] != this))
				{
					useLoop = true;
				}
				if (useLoop)
				{
					for (var i:int = 0; i < Math.abs(s.x); i++)
					{
						var v:Number = FP.sign(s.x) * Math.min(Math.abs(s.x) - i, 1);
						var c:Particle = collide("Solid", x+v, y) as Particle;
						if (x + v >= FP.camera.x && x + size + v < FP.camera.x + FP.width)
						{
							if (c)
							{
								c.s.x += v;
								if (!c.collide("Solid", c.x+v, c.y) && c.x + v >= FP.camera.x && c.x + c.size + v < FP.camera.x + FP.width)
								{
									c.x += v;
								}
								s.x -= v;
							}
							else
							{
								x += v;
							}
						}
						else
						{
							s.x = 0;
							break;
						}
					}
				}
				else
				{
					x += s.x;
					if (x + size > FP.camera.x + FP.width)
					{
						x = FP.camera.x + FP.width - size;
					}
					if (x < FP.camera.x)
					{
						x = FP.camera.x;
					}
				}
			}
			if (s.y != 0)
			{
				useLoop = false;
				ve = new Vector.<Entity>();
				var yPos:int = 0;
				if (s.y < 0)
				{
					yPos = s.y;
				}
				FP.world.collideRectInto("Solid", x, y +yPos, size, Math.abs(s.y) + size, ve);
				if (ve.length >= 2 || (ve.length > 0 && ve[0] != this))
				{
					useLoop = true;
				}
				if (useLoop)
				{
					for (i = 0; i < Math.abs(s.y); i++)
					{
						v = FP.sign(s.y) * Math.min(Math.abs(s.y) - i, 1);
						if(y + v + size < FP.camera.y + FP.height)
						{
							c = collide("Solid", x, y+v) as Particle;
							if (c)
							{
								c.s.y += v;
								if (!c.collide("Solid", c.x, c.y + v) && c.y + v + c.size < FP.camera.y + FP.height)
								{
									c.y += v;
								}
								s.y -= v;
							}
							else
							{
								y += v;
							}
						}
						else
						{
							s.y = 0;
							break;
						}
					}
				}
				else
				{
					y += s.y;
					if (y + size > FP.camera.y + FP.height)
					{
						y = FP.camera.y + FP.height - size;
					}
				}
			}
		}
		
		override public function render():void
		{
			Draw.rect(x, y, size, size, color);
			var lineCol:uint = 0x880000;
			Draw.line(x, y, x + size, y, lineCol);
			Draw.line(x, y, x, y + size, lineCol);
			Draw.line(x, y + size, x + size, y + size, lineCol);
			Draw.line(x + size, y, x + size, y + size, lineCol);
		}
		
	}

}